import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

public class Main implements GLEventListener {

    private Geom geom = new Geom();

    /** magnification factor */
    private static final float MAG = 1;

    Geom.V[] translations;
    Geom.P[] points;

    public Main(StlFile stlf) {

        Geom.P ltf = geom.newP(-0.2,  0.1,  0.1);
        Geom.P mtf = geom.newP( 0.0,  0.1,  0.1);
        Geom.P rtf = geom.newP( 0.2,  0.1,  0.1);
        Geom.P ltn = geom.newP(-0.2,  0.1, -0.1);
        Geom.P mtn = geom.newP( 0.0,  0.1, -0.1);
        Geom.P rtn = geom.newP( 0.2,  0.1, -0.1);
        Geom.P lbf = geom.newP(-0.2, -0.1,  0.1);
        Geom.P mbf = geom.newP( 0.0, -0.1,  0.1);
        Geom.P rbf = geom.newP( 0.2, -0.1,  0.1);
        Geom.P lbn = geom.newP(-0.2, -0.1, -0.1);
        Geom.P mbn = geom.newP( 0.0, -0.1, -0.1);
        Geom.P rbn = geom.newP( 0.2, -0.1, -0.1);
        
        points = new Geom.P[] {
            ltf,
            mtf,
            rtf,
            ltn,
            mtn,
            rtn,
            lbf,
            mbf,
            rbf,
            lbn,
            mbn,
            rbn
        };

        translations = new Geom.V[] {
            geom.new V(-0.2,  0.2,    0),
            geom.new V( 0.2,  0.2,    0),
            geom.new V(-0.2, -0.2,    0),
            geom.new V( 0.2, -0.2,    0),
            geom.new V( 0.4,    0,    0),
            geom.new V(-0.4,    0,    0),
            geom.new V(   0,    0,  0.2),
            geom.new V(   0,    0, -0.2),
        };

        for(Geom.V v : translations) {
            for(Geom.P p1 : points) {
                for(Geom.P p2 : points) {
                    if (p1.plus(v).minus(p2).mag() < Geom.EPSILON) {
                        System.out.println("bind " + p1 + " to " + p2);
                        p1.bind(p2);
                    }
                }
            }
        }

        // top
        geom.newT(ltf, mtf, mtn);
        geom.newT(mtn, ltn, ltf);
        geom.newT(mtf, rtf, rtn);
        geom.newT(rtn, mtn, mtf);

        // bottom (swap normals)
        geom.newT(mbf, lbf, mbn);
        geom.newT(lbn, mbn, lbf);
        geom.newT(rbf, mbf, rbn);
        geom.newT(mbn, rbn, mbf);
        
        // left
        geom.newT(ltf, ltn, lbn);
        geom.newT(lbn, lbf, ltf);

        // right (swap normals)
        geom.newT(rtn, rtf, rbn);
        geom.newT(rbf, rbn, rtf);

        // front
        geom.newT(ltn, mtn, mbn);
        geom.newT(ltn, mbn, lbn);
        geom.newT(mtn, rtn, rbn);
        geom.newT(mtn, rbn, mbn);

        // back
        geom.newT(mtf, ltf, mbf);
        geom.newT(mbf, ltf, lbf);
        geom.newT(rtf, mtf, rbf);
        geom.newT(rbf, mtf, mbf);


        /*
        for(int i=0; i<stlf.coordArray.length; i+=3) {
            Geom.P p0 = geom.newP(stlf.coordArray[i+0].x * MAG, stlf.coordArray[i+0].y * MAG, stlf.coordArray[i+0].z * MAG);
            Geom.P p1 = geom.newP(stlf.coordArray[i+1].x * MAG, stlf.coordArray[i+1].y * MAG, stlf.coordArray[i+1].z * MAG);
            Geom.P p2 = geom.newP(stlf.coordArray[i+2].x * MAG, stlf.coordArray[i+2].y * MAG, stlf.coordArray[i+2].z * MAG);
            Geom.V n  = geom.new V(stlf.normArray[i/3].x * MAG, stlf.normArray[i/3].y  * MAG, stlf.normArray[i/3].z * MAG);
            Geom.T t  = geom.newT(geom.newE(p0, p1), geom.newE(p1, p2), geom.newE(p2, p0), n);
        }
        */


        mtf.move(geom.new V(0, (float)-0.05, (float)0.05));

    }

    /**
     * Take care of initialization here.
     */
    public void init(GLAutoDrawable gld) {
        GL gl = gld.getGL();
        GLU glu = new GLU();
        gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
        gl.glClearDepth(1.0);
        gl.glViewport(0, 0, 500, 300);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);
        
        gl.glLoadIdentity();
        gl.glRotatef((float)45, 1, 1, 1);

        //glu.gluOrtho2D(0.0, 500.0, 0.0, 300.0);
        display(gld);
    }

    public static int angle = 0;

    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) { }
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }
    public void display(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glPointSize(5.0f);
        gl.glLoadIdentity();
        gl.glRotatef(angle, 1, 1, 1);

        gl.glBegin(GL.GL_TRIANGLES);
        draw(gl, true);
        gl.glEnd();
        
        for(Geom.V v1 : translations) {
            if (v1.z==0 && v1.y==0) continue;
            Geom.V v = v1.times((float)1.1);
            gl.glTranslatef(v.x, v.y, v.z);
            draw(gl, false);
            gl.glTranslatef(-v.x, -v.y, -v.z);
        }
    }

    private void draw(GL gl, boolean triangles) {
        float red = 0.0f;
        float green = 0.0f;
        float blue = 0.0f;
        for(Geom.T t : geom) {
            if (red < 0.15) red = 1.0f;
            if (green < 0.15) green = 1.0f;
            if (blue < 0.15) blue = 1.0f;
            red -= .09f;
            green -= .12f;
            blue -= .15f;
            gl.glColor3f(red, green, blue);
            //gl.glBegin(GL.GL_LINES);
            if (triangles) {
                gl.glBegin(GL.GL_TRIANGLES);
                t.glVertices(gl);
                gl.glEnd();
            } else {
                gl.glBegin(GL.GL_LINES);
                t.e1.p1.glVertex(gl);
                t.e1.p2.glVertex(gl);
                t.e2.p1.glVertex(gl);
                t.e2.p2.glVertex(gl);
                t.e3.p1.glVertex(gl);
                t.e3.p2.glVertex(gl);
                gl.glEnd();
            }
            /*
            Geom.P centroid = t.centroid();
            gl.glBegin(GL.GL_LINES);
            gl.glColor3f(1, 1, 1);
            centroid.glVertex(gl);
            centroid.plus(t.norm().times(t.diameter())).glVertex(gl);
            t.p1().glVertex(gl);
            t.p1().plus(t.p1().norm().times(t.diameter())).glVertex(gl);
            gl.glEnd();
            */
        }
    }

    public static void main(String[] s) throws Exception {
        StlFile stlf = new StlFile();
        stlf.load("teapot.stl");
        Main main = new Main(stlf);
        Frame f = new Frame();
        GLCapabilities glcaps = new GLCapabilities();
        GLCanvas glcanvas = new GLCanvas();
        glcanvas.addGLEventListener(main);
        f.add(glcanvas, BorderLayout.CENTER);
        f.show();
        f.setSize(500, 300);
        while(true) {
            Thread.sleep(100);
            angle+=5;
            glcanvas.repaint();
        }
    }

}